SECRET_KEY = "Thanks for using django-marina!"

DEFAULT_AUTO_FIELD = "django.db.models.BigAutoField"

DATABASES = {"default": {"ENGINE": "django.db.backends.sqlite3", "NAME": ":memory:"}}

INSTALLED_APPS = (
    # Default Django apps
    "django.contrib.admin",
    "django.contrib.auth",
    "django.contrib.contenttypes",
    "django.contrib.sessions",
    "django.contrib.messages",
    "django.contrib.staticfiles",
    # Our tests
    "django_marina",
    "tests",
)

ROOT_URLCONF = "tests.app.urls"
LOGIN_URL = "login"

MIDDLEWARE = (
    "django.contrib.sessions.middleware.SessionMiddleware",
    "django.contrib.auth.middleware.AuthenticationMiddleware",  # required for django.contrib.admin
    "django.contrib.messages.middleware.MessageMiddleware",  # required for django.contrib.admin
)

STATIC_URL = "/static/"

TEMPLATES = [
    {
        "BACKEND": "django.template.backends.django.DjangoTemplates",
        "APP_DIRS": True,
        "OPTIONS": {
            "context_processors": [
                "django.contrib.auth.context_processors.auth",
                "django.template.context_processors.request",
                "django.template.context_processors.debug",
                "django.template.context_processors.i18n",
                "django.template.context_processors.media",
                "django.template.context_processors.static",
                "django.template.context_processors.tz",
                "django.contrib.messages.context_processors.messages",
            ]
        },
    }
]



import os

try:
    from importlib.metadata import metadata
except ImportError:
    from importlib_metadata import metadata

PROJECT_NAME = "django-marina"

on_rtd = os.environ.get("READTHEDOCS", None) == "True"
project_metadata = metadata(PROJECT_NAME)

project = project_metadata["name"]
author = project_metadata["author"]
copyright = f"2020, {author}"

# The full version, including alpha/beta/rc tags, in x.y.z.misc format
release = project_metadata["version"]
# The short X.Y version.
version = ".".join(release.split(".")[:2])

extensions = ["sphinx.ext.autodoc", "sphinx.ext.viewcode", "m2r2"]
pygments_style = "sphinx"
htmlhelp_basename = f"{PROJECT_NAME}-doc"

if not on_rtd:  # only import and set the theme if we're building docs locally
    import sphinx_rtd_theme

    html_theme = "sphinx_rtd_theme"
    html_theme_path = [sphinx_rtd_theme.get_html_theme_path()]



from django.test import TestCase


class ImportDbTestCase(TestCase):
    def test_import_disable_migrations(self):
        from django_marina.db import DisableMigrations  # noqa

    def test_import_protected_model_mixin(self):
        from django_marina.db import ProtectedModelMixin  # noqa


class ImportTestTestCase(TestCase):
    def test_import_extended_client(self):
        from django_marina.test import ExtendedClient  # noqa

    def test_import_extended_test_case(self):
        from django_marina.test import ExtendedTestCase  # noqa



__all__ = [
    "__version__",
]

PACKAGE_NAME = "django-marina"

try:
    from importlib.metadata import metadata
except ImportError:
    from importlib_metadata import metadata

package_metadata = metadata(PACKAGE_NAME)
__version__ = package_metadata["Version"]



from django.urls import path

from .views import EchoView, HomeView, LoginRequiredView, LoginView, MessageView, SuperuserRequiredView, redirect_root

urlpatterns = [
    path("", HomeView.as_view(), name="home"),
    path("login/", LoginView.as_view(), name="login"),
    path("authenticated/", LoginRequiredView.as_view(), name="access_authenticated"),
    path("superuser/", SuperuserRequiredView.as_view(), name="access_superuser"),
    path("echo/", EchoView.as_view(), name="echo"),
    path("message/", MessageView.as_view(), name="message"),
    path("redirect-root/", redirect_root, name="redirect-root"),
]



from django.conf.urls import url
from django.contrib.auth.decorators import login_required
from django.core.exceptions import PermissionDenied
from django.http import HttpResponse


def _response(request):
    lines = [f"method: {request.method}", f"user: {request.user}"]
    return HttpResponse("\n".join(lines))


def echo(request):
    return _response(request)


@login_required
def logged_in_only(request):
    return _response(request)


def superuser_only(request):
    if request.user.is_superuser:
        return _response(request)
    raise PermissionDenied()


urlpatterns = [
    url(r"^echo$", echo, name="echo"),
    url(r"^logged_in_only$", logged_in_only, name="logged_in_only"),
    url(r"^superuser_only$", superuser_only, name="superuser_only"),
]



from django.contrib import messages
from django.contrib.auth.mixins import LoginRequiredMixin, PermissionRequiredMixin
from django.http import HttpResponse
from django.shortcuts import redirect
from django.utils.html import format_html
from django.utils.safestring import mark_safe
from django.views import View


class HomeView(View):
    html = "<html><head><title>Hello World!</title></head><body><h1 class='title'>Hello World!</h1></body></html>"

    def get(self, *args, **kwargs):
        return HttpResponse(self.html)

    def post(self, *args, **kwargs):
        return HttpResponse(self.html)


class LoginView(View):
    html = "<html><head><title>Login</title></head><body>Login</body></html>"

    def get(self, *args, **kwargs):
        return HttpResponse(self.html)

    def post(self, *args, **kwargs):
        return HttpResponse(self.html)


class LoginRequiredView(LoginRequiredMixin, HomeView):
    pass


class SuperuserRequiredView(PermissionRequiredMixin, HomeView):
    def has_permission(self):
        return self.request.user.is_superuser


class EchoView(View):
    def _response(self, data):
        return HttpResponse(format_html("<html><head><title>Echo</title></head><body>{data}</body></html>", data=data))

    def get(self, *args, **kwargs):
        return self._response(data=self.request.GET)

    def post(self, *args, **kwargs):
        return self._response(data=self.request.POST)


class MessageView(View):
    def get(self, *args, **kwargs):
        message = self.request.GET.get("message", "")
        if message:
            messages.add_message(self.request, messages.INFO, message)
        return HttpResponse(mark_safe("<html><head><title>Message</title></head><body>Message</body></html>"))


def redirect_root(request):
    """Redirects to root url."""
    return redirect("/")



import logging

logger = logging.getLogger(__name__)


class DisableMigrations(object):
    """
    Settings class to disable migrations.

    In your `settings_test.py` you can set MIGRATION_MODULES, for example:

    .. code-block:: python

      from django_marina.db import DisableMigrations

      if some_condition:
          MIGRATION_MODULES = DisableMigrations()
    """

    # Based on: https://simpleisbetterthancomplex.com/tips/2016/08/19/django-tip-12-disabling-migrations-to-speed-up-unit-tests.html  # NOQA
    def __init__(self, *args, **kwargs):
        logger.debug("Migrations disabled.")

    def __contains__(self, item):
        return True

    def __getitem__(self, item):
        return None



from django.db import models

from django_marina.db import ProtectedModelMixin


class ProtectedModel(ProtectedModelMixin, models.Model):
    name = models.CharField(max_length=10)
    is_update_protected = models.BooleanField(default=True)
    is_delete_protected = models.BooleanField(default=True)

    @property
    def has_update_protection(self):
        if self.is_update_protected:
            return True
        return super().has_update_protection

    @property
    def has_delete_protection(self):
        if self.is_delete_protected:
            return True
        return super().has_delete_protection
